#include <conio.h>        // cekani na stisk klavesy, ...
#include <graphics.h>     // pro praci v grafickem rezimu pri zobrazeni fce
#include <math.h>         // pro matematicke operace (odmocnina apod.)
#include "g_graph.h"

#define DelkaUsecky 3     // pro presnost vykresleni

typedef struct {
  float x,y;
} TBod;

/*** staticke funkce (viditelne pouze v tomto modulu): ***/

// inicializace grafiky, zjisteni koeficientu pro transformaci souradnic, ...:
static int inicializace(TFce fce);
// vypocet hodnoty funkce:
static float HodnotaFce(TFce fce, float x);
// zaokrouhleni racionalniho cisla na cele cislo:
static int zaokrouhli(float f);
// vypocet vzdalenosti dvou bodu vcetne transformace:
static float vzdalenost(TBod b1, TBod b2);
// vykresli usecku danou dvema krajnimi body:
static void VykresliUsecku(TFce f, TBod b1, TBod b2);
// vykresli funkci v intervalu zadanem dvema body (rekurzivne):
static void VykresliMezi(TFce f, TBod b1, TBod b2);

/*** staticke promenne (viditelne pouze v tomto modulu),
     koeficienty pro transformaci: ***/

static float indexX;
static float indexY;

/*** definice funkci: ***/

void vykresli(TFce f)
{
  TBod b1, b2;

  if (!inicializace(f)) {  // jestlize se nepodari inicializovat grafiku:
    textbackground(BLACK);
    textcolor(YELLOW);
    clrscr();
    cputs("Bohuel se nepodailo nastartovat grafiku. Funkci nelze zobrazit.");
    getch();
  } else {
    b1.x = f.x1;  b1.y = HodnotaFce(f, b1.x); // hranicni body funkce
    b2.x = f.x2;  b2.y = HodnotaFce(f, b2.x);
    if (b1.x < b2.x) VykresliMezi(f, b1, b2); // pokud ma interval vetsi delku nez 0
    getch();               // po vykresleni pocka na stisk jakekoliv klavesy
    closegraph();
  }
}

int zaokrouhli(float f)
{
  int i = f;    // odstrani, odsekne desetinnou cast
  if ((f - i) < 0.5) return i;
  return(i+1);
}

float HodnotaFce(TFce f, float x)
{
  return (f.a * x*x*x + f.b * x*x + f.c * x + f.d);
}

float vzdalenost(TBod b1, TBod b2)
{
  float dx, dy, mx, my;
  // vypocte vzdalenost dvou bodu, kdyz jsou zobrazeny na obrazovce (provede transformaci)
  dx = b1.x - b2.x;
  dy = b1.y - b2.y;
  mx = indexX*indexX;
  my = indexY*indexY;
  return (sqrt(mx*dx*dx + my*dy*dy));
}

void VykresliUsecku(TFce f, TBod b1, TBod b2)
{       // najde umisteni usecky na obrazovce (transformace) a vykresli ji
  int x1, x2, y1, y2;
  int mx, my;

  mx = getmaxx();
  my = getmaxy();
  x1 = zaokrouhli((abs(f.x1) + b1.x) * indexX);
  if((x1 < 0)||(x1 > mx)) return; // krajni bod mimo zobrazitelnou oblast

  x2 = zaokrouhli((abs(f.x1) + b2.x) * indexX);
  if((x2 < 0)||(x2 > mx)) return; // krajni bod mimo zobrazitelnou oblast

  y1 = zaokrouhli((abs(f.y2) - b1.y) * indexY);
  if((y1 < 0)||(y1 > my)) return; // krajni bod mimo zobrazitelnou oblast

  y2 = zaokrouhli((abs(f.y2) - b2.y) * indexY);
  if((y2 < 0)||(y2 > my)) return; // krajni bod mimo zobrazitelnou oblast

  line(x1, y1, x2, y2);
}

void VykresliMezi(TFce f, TBod b1, TBod b2)
{
  TBod mez;

  if((vzdalenost(b1,b2)) < DelkaUsecky) { // rekurzivni funkce pro vykresleni
    VykresliUsecku(f, b1, b2);            // casti funkce mezi dvema body
    return;                      // - pokud je jejich vzdalenost
  }  // jinak:                   //   mensi nez zadana konstanta, vykresli;
                                 // - jinak rozdeli na dve casti a znovu
  mez.x = (b1.x+b2.x) / 2;       //   vola sama sebe pro tyto casti
  mez.y = HodnotaFce(f, mez.x);
  VykresliMezi(f, b1, mez);
  VykresliMezi(f, mez, b2);
}

int inicializace(TFce fce)
{
  int gdriver, gmode;
  int sx,sy;          // souradnice stredu na obrazovce (pro osy x a y)

  gdriver = DETECT;   // zjistime parametry grafiky
  initgraph(&gdriver, &gmode, "");
  if((graphresult()) != grOk)
    return 0;         //pokud jsme nenastartovali grafiku, odchod

  indexX = 1.0 * getmaxx() / (fce.x2 - fce.x1); // vypocet indexu pro
  indexY = 1.0 * getmaxy() / (fce.y2 - fce.y1); // transformaci souradnic
  sx = -(zaokrouhli(fce.x1)) * indexX;
  sy = (zaokrouhli(fce.y2)) * indexY;

  if((fce.x1 < 0.0)&&(0.0 < fce.x2)) line(sx,0,sx,(getmaxy()));  // osa y
  if((fce.y1 < 0.0)&&(0.0 < fce.y2)) line(0,sy,(getmaxx()),sy);  // osa x

  setcolor(fce.barva);
  return 1;
}

